import bb.cascades 1.0
import Network.FeedHttpCalls 1.0
import org.labsquare 1.0

Page {
    property bool loggedInActivity: loggedIn
    property bool allowAtEndSignal
    id: activityView
    titleBar: TitleBar {
        id: titleBar
        scrollBehavior: TitleBarScrollBehavior.Sticky
        title: qsTr("Activity")
        kind: TitleBarKind.FreeForm
        kindProperties: FreeFormTitleBarKindProperties {
            content: Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                background: Color.create("#FF6500")
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    topPadding: 25
                    leftPadding: 15
                    bottomPadding: 10
                    preferredWidth: maxWidth

                    Label {
                        text: "Activity"
                        textStyle.fontSize: FontSize.Large
                        textStyle.color: Color.White
                    }
                }
                Container {
                    horizontalAlignment: HorizontalAlignment.Right
                    verticalAlignment: VerticalAlignment.Center
                    rightPadding: 15
                    Container {
                        layout: DockLayout {
                        }
                        Container {
                            id: outer
                            background: Color.Transparent
                            topPadding: 3
                            bottomPadding: 3
                            rightPadding: 3
                            leftPadding: 4
                            WebImageView {
                                property variant blueBackground: Color.create("#00A7DD")
                                horizontalAlignment: HorizontalAlignment.Right
                                verticalAlignment: VerticalAlignment.Center
                                id: nowPlayingThumb
                                preferredHeight: 85
                                preferredWidth: 85
                                minWidth: 85
                                enabled: false
                                scalingMethod: ScalingMethod.AspectFit
                                url: nowPlaying.artwork_url
                                onTouch: {
                                    if (enabled) {
                                        if (event.touchType == TouchType.Down) {
                                            outer.background = blueBackground;
                                        } else if (event.touchType == TouchType.Up) {
                                            outer.background = Color.Transparent;
                                            app.removePageParent(mediaView);
                                            navPane4.push(mediaView);
                                        } else if (event.touchType == TouchType.Cancel) {
                                            outer.background = Color.Transparent;
                                        }
                                    }
                                }
                                onUrlChanged: {
                                    if (url) {
                                        enabled = true;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    Container {
        layout: DockLayout {
        }
        ListView {
            scrollRole: ScrollRole.Main
            objectName: "feedList"
            id: feedList
            property bool loading: false
            leadingVisualSnapThreshold: 2.0
            leadingVisual: RefreshHeader {
                id: refreshHandler
                onRefreshTriggered: {
                    feedHttp.getActivity(activity, feedList);
                }
            }
            onTouch: {
                refreshHandler.onListViewTouch(event);
            }
            onLoadingChanged: {
                refreshHandler.refreshing = feedList.loading;
                
                if(!refreshHandler.refreshing) {
                    scrollToPosition(ScrollPosition.Beginning, ScrollAnimation.Smooth);
                }
            }
            listItemComponents: [
                ListItemComponent {
                    id: listItemRoot
                    type: "affiliation"
                    ActivityListItem {
                        id: activityListItem
                    }
                },
                ListItemComponent {
                    id: commentListComponent
                    type: "comment"
                    CommentListItem {
                        id: commentListItem
                    }
                },
                ListItemComponent {
                    id: footerListItem
                    type: "footer"
                    Container {
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        id: footerContainer
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        ActivityIndicator {
                            running: true
                            preferredWidth: 100
                            preferredHeight: 100
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                        }
                        Label {
                            verticalAlignment: VerticalAlignment.Center
                            text: "loading more"
                        }
                    }
                }
            ] //ListItemComponents
            function itemType(data, indexPath) {
                var currentItem = dataModel.data(indexPath);
                if (currentItem.type == "comment") {
                    return "comment";
                } else if (currentItem.type == "footer") {
                    return "footer";
                } else return "affiliation"
            }
            onTriggered: {
                var selectedItem = dataModel.data(indexPath);
                var page = userView.createObject();
                page.userID = selectedItem.user.id;
                //page.artworkUrl = getNowPlayingArtwork();
                navPane4.push(page);
            }
            onDataModelChanged: {
                if (dataModel.size() <= 2) {
                    allowAtEndSignal = false;
                    emptyLabel.setVisible(true);
                } else {
                    allowAtEndSignal = true;
                    emptyLabel.setVisible(false);
                }
            }
            function pushPageStack(pageParam) {
                navPane4.push(pageParam);
            }
            function getNowPlayingArtwork() {
                if (nowPlaying) {
                    return nowPlaying.artwork_url;
                } else return ""
            }
            onCreationCompleted: {
                if (loggedInActivity) {
                    feedHttp.getActivity(activity, feedList);
                }
            }
            attachedObjects: [
                ComponentDefinition {
                    id: userView
                    source: "UserView.qml"
                },
                ListScrollStateHandler {
                    onAtEndChanged: {
                        if (atEnd == true && allowAtEndSignal == true) {
                            allowAtEndSignal = false;
                            feedHttp.getNextPage(feedList);
                        }
                    }
                }
            ]
        }
        ActivityIndicator {
            objectName: "indicator"
            id: activity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
        Label {
            id: emptyLabel
            visible: false
            text: "there doesn't seem to be anything here"
            textStyle.color: Color.Red
            multiline: true
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            leftMargin: 20
            rightMargin: 20
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onNextPageFinished: {
                allowAtEndSignal = true;
            }
        }
    ]
    onLoggedInActivityChanged: {
        feedHttp.getActivity(activity, feedList);
    }
}
