import bb.cascades 1.0
import bb.system 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0
import org.labsquare 1.0

Page {
    property bool loggedInFeed: loggedIn
    property bool allowAtEndSignal
    id: feedView
    titleBar: TitleBar {
        id: titleBar
        scrollBehavior: TitleBarScrollBehavior.Sticky
        title: qsTr("Stream")
        kind: TitleBarKind.FreeForm
        kindProperties: FreeFormTitleBarKindProperties {
            content: Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                background: Color.create("#FF6500")
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    topPadding: 25
                    leftPadding: 15
                    bottomPadding: 10
                    preferredWidth: maxWidth
                    Label {
                        text: "Stream"
                        textStyle.fontSize: FontSize.Large
                        textStyle.color: Color.White
                    }
                }
                Container {
                    horizontalAlignment: HorizontalAlignment.Right
                    verticalAlignment: VerticalAlignment.Center
                    rightPadding: 15
                    Container {
                        layout: DockLayout {
                        }
                        Container {
                            id: outer
                            background: Color.Transparent
                            topPadding: 3
                            bottomPadding: 3
                            rightPadding: 3
                            leftPadding: 4
                            WebImageView {
                                property variant blueBackground: Color.create("#00A7DD")
                                horizontalAlignment: HorizontalAlignment.Right
                                verticalAlignment: VerticalAlignment.Center
                                id: nowPlayingThumb
                                preferredHeight: 85
                                preferredWidth: 85
                                minWidth: 85
                                enabled: false
                                scalingMethod: ScalingMethod.AspectFit
                                url: nowPlaying.artwork_url
                                onTouch: {
                                    if (enabled) {
                                        if (event.touchType == TouchType.Down) {
                                            outer.background = blueBackground;
                                        } else if (event.touchType == TouchType.Up) {
                                            outer.background = Color.Transparent;
                                            app.removePageParent(mediaView);
                                            navPane.push(mediaView);
                                        } else if (event.touchType == TouchType.Cancel) {
                                            outer.background = Color.Transparent;
                                        }
                                    }
                                }
                                onUrlChanged: {
                                    if (url) {
                                        enabled = true;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    Container {
        layout: DockLayout {
        }
        ListView {
            scrollRole: ScrollRole.Main
            objectName: "feedList"
            id: feedList
            property bool loading: false
            leadingVisualSnapThreshold: 2.0
            leadingVisual: RefreshHeader {
                id: refreshHandler
                onRefreshTriggered: {
                    feedHttp.getFeed(activity, feedList);
                }
            }
            onTouch: {
                refreshHandler.onListViewTouch(event);
            }
            onLoadingChanged: {
                refreshHandler.refreshing = feedList.loading;

                if (! refreshHandler.refreshing) {
                    scrollToPosition(ScrollPosition.Beginning, ScrollAnimation.Smooth);
                }
            }
            listItemComponents: [
                ListItemComponent {
                    id: listItemRoot
                    type: "track"
                    FeedListItem {
                        id: feedListItem
                    }
                },
                ListItemComponent {
                    type: "trackRepost"
                    FeedListRepostItem {
                        id: feedListRepostItem
                    }
                },
                ListItemComponent {
                    type: "playlist"
                    PlaylistListItem {
                        id: playlistListItem
                    }
                },
                ListItemComponent {
                    type: "playlist-repost"
                    PlaylistRepostListItem {
                        id: playlistRepostListItem
                    }
                },
                ListItemComponent {
                    id: footerListItem
                    type: "footer"
                    Container {
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        id: footerContainer
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        ActivityIndicator {
                            running: true
                            preferredWidth: 100
                            preferredHeight: 100
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                        }
                        Label {
                            verticalAlignment: VerticalAlignment.Center
                            text: "loading more"
                        }
                    }
                },
                ListItemComponent {
                    type: "none"
                    Container {
                        visible: false
                    }
                }
            ] //ListItemComponents
            function itemType(data, indexPath) {
                var currentItem = dataModel.data(indexPath);
                if (currentItem.type == "footer") {
                    return "footer";
                } else if (currentItem.type == "track-repost") {
                    return "trackRepost"
                } else if (currentItem.type == "track") {
                    return "track"
                } else if (currentItem.type == "playlist") {
                    return "playlist";
                } else if (currentItem.type == "playlist-repost") {
                    return "playlist-repost";
                } else return "none";
            }
            onDataModelChanged: {
                if (dataModel.size() <= 0) {
                    allowAtEndSignal = false;
                    emptyLabel.setVisible(true);
                } else {
                    allowAtEndSignal = true;
                    emptyLabel.setVisible(false);
                }
            }
            function pushPageStack(pageParam) {
                navPane.push(pageParam);
            }
            function getNowPlayingArtwork() {
                if (nowPlaying) {
                    return nowPlaying.artwork_url;
                } else return ""
            }
            function downloadTrack(url) {
                console.log("feedView dl url -- " + url);
                feedHttp.getDownloadUrl(url);
            }
            onCreationCompleted: {
                if (loggedInFeed) {
                    feedHttp.getFeed(activity, feedList);
                }
            }
            onTriggered: {
                var selectedItem = dataModel.data(indexPath);
                if (selectedItem.type == "playlist" || selectedItem.type == "playlist-repost") {
                	var playlistFeed = playlistView.createObject();
                	playlistFeed.playlistID = selectedItem.playlist.id;
                	navPane.push(playlistFeed); 
                } else {
                    selectedItem = selectedItem.track;
                    if (selectedItem.streamable == false) {
                        app.showStreamableToast(selectedItem.title);
                        return;
                    }
                    currentDB = dataModel;
                    currentIndex = indexPath[0];
                    if (nowPlaying == undefined) {
                        navPane.push(mediaView);
                    }
                    if (selectedItem.artwork_url == null) {
                        selectedItem.artwork_url = selectedItem.user.avatar_url;
                    }
                    nowPlaying = selectedItem;
                }
            }
            attachedObjects: [
                ListScrollStateHandler {
                    onAtEndChanged: {
                        if (atEnd == true && allowAtEndSignal == true) {
                            allowAtEndSignal = false;
                            feedHttp.getNextPage(feedList);
                        }
                    }
                },
                ComponentDefinition {
                    id: playlistView
                    source: "PlaylistView.qml"
                }
            ]
        }
        ActivityIndicator {
            objectName: "indicator"
            id: activity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
        Label {
            id: emptyLabel
            visible: false
            text: "there doesn't seem to be anything here"
            textStyle.color: Color.Red
            multiline: true
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            leftMargin: 20
            rightMargin: 20
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onNextPageFinished: {
                allowAtEndSignal = true;
            }
        }
    ]
    onLoggedInFeedChanged: {
        feedHttp.getFeed(activity, feedList);
    }
}
