import bb.cascades 1.0
import bb.system 1.0
import bb.multimedia 1.0
import bb.data 1.0
import org.labsquare 1.0
import org.domisy 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Page {
    id: mediaView
    property int displayWidth: app.getDisplayWidth()
    property string streamUrl: nowPlaying.stream_url
    property string length
    property bool isPanning
    property double bufferPercent
    property int index: currentIndex
    property alias audioPlayer: audioPlayer
    property alias nowPlayingConnect: nowPlayingConnect
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            onTriggered: {
                mediaView.parent.pop()
            }
            title: "Back"
        }
    }
    actionBarVisibility: ChromeVisibility.Visible
    actions: [
        ActionItem {
            id: previousAction
            title: "Previous"
            enabled: checkRevEnabled()
            imageSource: "asset:///images/reverseButtonx81.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                previousTrack();
            }
            function checkRevEnabled() {
                if (index < 1) {
                    return false;
                } else {
                    return true;
                }
            }
        },
        ActionItem {
            id: playAction
            title: "Play"
            imageSource: "asset:///images/playButtonx81.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                if (audioPlayer.mediaState == MediaState.Paused) {
                    audioPlayer.play();
                    setImageSource("asset:///images/pauseButtonx81.png")
                    title = "Pause"
                } else if (audioPlayer.mediaState == MediaState.Started) {
                    audioPlayer.pause();
                    setImageSource("asset:///images/playButtonx81.png")
                    title = "Play"
                }
            }
        },
        ActionItem {
            id: nextAction
            title: "Next"
            enabled: checkForEnabled()
            imageSource: "asset:///images/forwardButtonx81.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                nextTrack();
            }
            function checkForEnabled() {
                if (index < 0 || currentDB.data([ index + 2 ]) == null) {
                    return false;
                } else {
                    return true;
                }
            }
        },
        ActionItem {
            id: likeAction
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: checkLike()
            imageSource: "asset:///images/Heart.png"
            onTriggered: {
                //var selectedItem = nowPlaying;
                var dir;
                if (title == "Like") {
                    dir = "PUT"
                    nowPlaying.user_favorite = true;
                    likeAction.setImageSource("asset:///images/HeartPressed.png");
                    likeAction.setTitle("Unlike");
                } else {
                    dir = "DELETE"
                    nowPlaying.user_favorite = false;
                    likeAction.setImageSource("asset:///images/Heart.png");
                    likeAction.setTitle("Like");
                }
                postActions.likeTrack(nowPlaying.id, dir);
            }
            function checkLike() {
                if (nowPlaying.user_favorite == true) {
                    likeAction.setImageSource("asset:///images/HeartPressed.png");
                    return "Unlike";
                } else {
                    likeAction.setImageSource("asset:///images/Heart.png");
                    return "Like";
                }
            }
        },
        ActionItem {
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/repostContextIconx81.png"
            title: "Repost"
            onTriggered: {
                postActions.repostTrack(nowPlaying.id);
            }
        },
        ActionItem {
            title: "Add Comment"
            imageSource: "asset:///images/add_comment.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            onTriggered: {
                newCommentPrompt.trackForComment = nowPlaying.id;
                newCommentPrompt.timeStamp = audioPlayer.position;
                newCommentPrompt.show();
            }
            attachedObjects: [
                SystemPrompt {
                    property string trackForComment
                    property string timeStamp
                    id: newCommentPrompt
                    title: "New comment at " + audioPlayer.parseTime(timeStamp)
                    inputField.defaultText: "Write a comment..."
                    onFinished: {
                        if (result == SystemUiResult.ConfirmButtonSelection) {
                            postActions.createComment(trackForComment, timeStamp, inputFieldTextEntry());
                        }
                    }
                }
            ]
        },
        ActionItem {
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/addToPlaylist.png"
            title: "Add to Playlist"
            onTriggered: {
                postActions.showPlaylistDialog(activity, nowPlaying.id);
            }
        },
        InvokeActionItem {
            ActionBar.placement: ActionBarPlacement.InOverflow
            id: shareAction
            objectName: "shareAction"
            title: "Share"
            query {
                mimeType: "text/plain"
                invokeActionId: "bb.action.SHARE"
            }
            onTriggered: {
                data = nowPlaying.title + " -- " + nowPlaying.permalink_url + "\nvia SoundNine";
            }
        }
    ]
    Container {
        layout: DockLayout {

        }
        Container {
            background: Color.DarkGray
            layout: StackLayout {

            }
            Container {
                layout: StackLayout {

                }
                Container {
                    preferredWidth: maxWidth
                    background: Color.create("#FF6500")
                    horizontalAlignment: HorizontalAlignment.Center
                    Container {
                        leftPadding: 20
                        topPadding: 27
                        bottomPadding: 27
                        Label {
                            text: "Now Playing"
                            textStyle.fontSize: FontSize.Large
                            textStyle.color: Color.White
                            bottomMargin: 0
                        }
                    }
                }
                Container {
                    //background: Color.Green
                    layout: DockLayout {
                    }
                    topPadding: 10
                    Container {
                        //background: Color.Yellow
                        layout: DockLayout {
                        }
                        //verticalAlignment: VerticalAlignment.Top
                        topPadding: 100
                        ImageView {
                            topMargin: 0
                            rightMargin: 0
                            leftMargin: 0
                            horizontalAlignment: HorizontalAlignment.Fill
                            verticalAlignment: VerticalAlignment.Fill
                            preferredHeight: displayWidth
                            preferredWidth: displayWidth
                            minWidth: displayWidth
                            minHeight: displayWidth
                            scalingMethod: ScalingMethod.AspectFit
                            imageSource: "asset:///images/noAlbumArtwork.png"
                        }
                        WebImageView {
                            topMargin: 0
                            rightMargin: 0
                            leftMargin: 0
                            horizontalAlignment: HorizontalAlignment.Fill
                            verticalAlignment: VerticalAlignment.Fill
                            id: nowPlayingThumb
                            preferredHeight: displayWidth
                            preferredWidth: displayWidth
                            minWidth: displayWidth
                            minHeight: displayWidth
                            scalingMethod: ScalingMethod.AspectFit
                            url: getLargeArtwork()
                            function getLargeArtwork() {
                                var thumbSize = nowPlaying.artwork_url;
                                return thumbSize.replace("large", "t500x500")

                            }
                        }
                        Container {
                            id: commentBox
                            property bool hidden: false
                            visible: false
                            layout: DockLayout {
                            }
                            verticalAlignment: VerticalAlignment.Bottom
                            minHeight: 50
                            Container {
                                verticalAlignment: VerticalAlignment.Fill
                                horizontalAlignment: HorizontalAlignment.Fill
                                background: Color.create('#333333')
                                opacity: 0.9
                            }
                            Container {
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                leftPadding: 15
                                rightPadding: 15
                                bottomPadding: 15
                                topPadding: 15
                                Container {
                                    ImageView {
                                        id: addCommentButton
                                        visible: false
                                        imageSource: "asset:///images/iconx114.png"
                                        preferredHeight: 100
                                        preferredWidth: 100
                                        minHeight: 100
                                        minWidth: 100
                                    }
                                    WebImageView {
                                        id: commentAvatarImage
                                        preferredHeight: 100
                                        preferredWidth: 100
                                        minHeight: 100
                                        minWidth: 100
                                    }
                                }
                                Container {
                                    leftPadding: 20
                                    Label {
                                        bottomMargin: 0
                                        id: commentAuthorLabel
                                        multiline: true
                                        //text: "SoundNine"
                                        textStyle.color: Color.create('#FF4727')
                                    }
                                    Label {
                                        topMargin: 5
                                        id: commentBodyLabel
                                        multiline: true
                                        content.flags: TextContentFlag.ActiveText
                                        text: checkCommentCount()
                                        textStyle.color: Color.White
                                        function checkCommentCount() {
                                            if (nowPlaying.comment_count == 0) {
                                                commentAuthorLabel.setText("SoundNine");
                                                commentAvatarImage.setVisible(false);
                                                addCommentButton.setVisible(true);
                                                if (! commentBox.hidden) commentBox.visible = true;
                                                return "Be the first to drop a comment!";
                                            } else {
                                                commentAuthorLabel.setText("");
                                                commentAvatarImage.setVisible(true);
                                                addCommentButton.setVisible(false);
                                                return "";
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        gestureHandlers: [
                            TapHandler {
                                onTapped: {
                                    commentBox.hidden = ! commentBox.hidden;
                                    if (commentBox.hidden) {
                                        commentBox.setVisible(false);
                                        commentToggleToast.body = "comments off";
                                        commentToggleToast.show();
                                    } else {
                                        commentBox.setVisible(true);
                                        commentToggleToast.body = "comments on";
                                        commentToggleToast.show();
                                    }
                                }
                            }
                        ]
                        attachedObjects: [
                            SystemToast {
                                id: commentToggleToast
                                body: "comments off"
                            }
                        ]
                    }
                    Container {
                        layout: StackLayout {
                        }
                        layoutProperties: StackLayoutProperties {
                            spaceQuota: 1.0
                        }
                        bottomMargin: 0
                        bottomPadding: 0
                        leftPadding: 10
                        rightPadding: 10
                        ImageView {
                            topMargin: 0
                            imageSource: "asset:///images/titleBar.png"
                            verticalAlignment: VerticalAlignment.Top
                            horizontalAlignment: HorizontalAlignment.Center
                            scalingMethod: ScalingMethod.Fill
                            minHeight: 150
                            //maxHeight: 20
                        }
                    }
                    Container {
                        preferredWidth: maxWidth
                        topPadding: 20
                        bottomPadding: 20
                        leftPadding: 25
                        rightPadding: 90
                        Label {
                            text: nowPlaying.title
                            textStyle.color: Color.Black
                            textStyle.fontSizeValue: 9
                            textStyle.fontSize: FontSize.PointValue
                        }
                    }
                    Container {
                        topPadding: 8
                        rightPadding: 10
                        horizontalAlignment: HorizontalAlignment.Right
                        ImageButton {
                            defaultImageSource: "asset:///images/infoNowPlaying.png"
                            pressedImageSource: "asset:///images/infoNowPlayingPressed.png"
                            onClicked: {
                                var page = infoView.createObject();
                                mediaView.parent.push(page);
                                //navPane.push(page);
                            }
                            attachedObjects: [
                                ComponentDefinition {
                                    id: infoView
                                    source: "InfoView.qml"
                                }
                            ]
                        }
                    }
                    ImageView {
                        imageSource: "asset:///images/dropShadow.png"
                        verticalAlignment: VerticalAlignment.Bottom
                        horizontalAlignment: HorizontalAlignment.Fill
                        maxHeight: 15
                        bottomMargin: 0
                    }
                }
            }
            Container {
                //bottomPadding: 140
                layout: DockLayout {
                }
                verticalAlignment: VerticalAlignment.Bottom
                id: waveformContainer
                horizontalAlignment: HorizontalAlignment.Fill
                Container {
                    verticalAlignment: VerticalAlignment.Fill
                    Container {
                        topPadding: 0
                        layout: DockLayout {
                        }
                        horizontalAlignment: HorizontalAlignment.Fill
                        background: Color.create('#959595') //313131
                        Container {
                            layout: DockLayout {
                            }
                            id: bufferProgressBar
                            background: Color.create('#363636')
                            opacity: 0.8
                            horizontalAlignment: HorizontalAlignment.Left
                            verticalAlignment: VerticalAlignment.Fill
                            touchPropagationMode: TouchPropagationMode.PassThrough
                            preferredWidth: 0
                        }
                        Container {
                            layout: DockLayout {
                            }
                            id: progressBar
                            background: Color.create('#FF5300')
                            opacity: 0.8
                            horizontalAlignment: HorizontalAlignment.Left
                            verticalAlignment: VerticalAlignment.Fill
                            touchPropagationMode: TouchPropagationMode.PassThrough
                            //preferredWidth: 100
                        }
                        WebImageView {
                            property string waveUrl: nowPlaying.waveform_url
                            topMargin: 0
                            rightMargin: 0
                            leftMargin: 0
                            horizontalAlignment: HorizontalAlignment.Fill
                            verticalAlignment: VerticalAlignment.Fill
                            id: waveform
                            //preferredHeight: 200
                            preferredWidth: maxWidth
                            scalingMethod: ScalingMethod.Fill
                            //url: nowPlaying.waveform_url
                            onWaveUrlChanged: {
                                if (waveUrl.indexOf("//wis") != -1) {
                                    waveform.url = waveUrl.replace("//wis", "//w1");
                                } else {
                                    waveform.url = waveUrl
                                }
                            }
                        }
                        Container {
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            //topPadding: 50
                            Label {
                                id: timeLabel
                                horizontalAlignment: HorizontalAlignment.Center
                                verticalAlignment: VerticalAlignment.Center
                                text: "buffering..."
                                textStyle.color: Color.White
                            }
                        }
                        onTouch: {
                            if (event.touchType == TouchType.Move) {
                                isPanning = true;
                                mediaView.parent.peekEnabled = false;
                                if (event.localX > 0) {
                                    progressBar.setPreferredWidth(event.localX)
                                }
                            } else if (event.touchType == TouchType.Up) {
                                mediaView.parent.peekEnabled = true;
                                var percent = event.localX / displayWidth
                                if (percent > 0) {
                                    bufferPercent = 0.0;
                                    audioPlayer.seekPercent(percent);
                                }
                                isPanning = false;
                            } else {
                                mediaView.parent.peekEnabled = true;
                                isPanning = false;
                            }
                        }
                    }
                }
            }
        }
        ActivityIndicator {
            id: activity
            horizontalAlignment: HorizontalAlignment.Center
            verticalAlignment: VerticalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
    }
    attachedObjects: [
        MediaPlayer {
            id: audioPlayer
            onPositionChanged: {
                if (! isPanning) {
                    var percentPlayed = position / duration;
                    if (percentPlayed > 0.001) {
                        progressBar.setPreferredWidth(percentPlayed * displayWidth); //720
                        timeLabel.setText(parseTime(position) + length);
                        nowPlayingConnect.position = position;
                        if (commentDataModel.isPopulated) {
                            commentCurrentTime(position);
                        }

                    }
                }

            }
            function commentCurrentTime(ms) {
                for (var i = 0; i < commentDataModel.size(); i ++) {
                    var localtimestamp = commentDataModel.value(i).timestamp;
                    if (localtimestamp != null && localtimestamp != 0 && localtimestamp < ms && (localtimestamp + 1000) > ms) {
                        if (commentQueue.indexOf(commentDataModel.value(i)) == -1)
                            commentQueue.append(commentDataModel.value(i));
                        //console.log("comment -- " + commentDataModel.value(i).body);
                    }
                }
            }
            onPlaybackCompleted: {
                if (nextAction.enabled) {
                    mediaView.nextTrack();
                }
            }
            onBufferStatusChanged: {
                if (bufferStatus == BufferStatus.Buffering) {
                    timeLabel.setText("buffering...");
                }
            }
            onBuffering: {
                if (percentage > bufferPercent) {
                    bufferProgressBar.setPreferredWidth(percentage * displayWidth);
                    bufferPercent = percentage;
                } else {
                    bufferPercent = 1.0;
                    bufferProgressBar.setPreferredWidth(displayWidth);
                }
                //bufferProgressBar.setPreferredWidth(percentage * displayWidth);
            }
            onMediaStateChanged: {
                if (mediaState == MediaState.Started) {
                    nowPlayingConnect.mediaState = MediaState.Started;
                    nowPlayingConnect.acquire();
                } else {
                    nowPlayingConnect.mediaState = MediaState.Paused;
                }
            }
            onDurationChanged: {
                var ms = duration
                var h, m, s;
                s = Math.floor(ms / 1000);
                m = Math.floor(s / 60);
                s = s % 60;
                h = Math.floor(m / 60);
                m = m % 60;
                //d = Math.floor(h / 24);
                h = h % 24;
                if (s < 10) {
                    s = "0" + s
                }
                length = " / ";
                if (h != 0) length += h + ":"
                length += m + ":" + s
                timeLabel.setText("0:00" + length);
            }
            function parseTime(ms) {
                var h, m, s, time;
                s = Math.floor(ms / 1000);
                m = Math.floor(s / 60);
                s = s % 60;
                h = Math.floor(m / 60);
                m = m % 60;
                h = h % 24;
                time = ""
                if (s < 10) {
                    s = "0" + s
                }
                if (h != 0) time += h + ":"
                time += m + ":" + s
                return time;
            }
        },
        FeedHttpCalls {
            id: feedHttp
            onStreamUrlFound: {
                nowPlayingConnect.acquire();
                bufferPercent = 0.0;
                audioPlayer.setSourceUrl(streamUrlP);
                audioPlayer.play();
                playAction.setImageSource("asset:///images/pauseButtonx81.png");
                playAction.setTitle("Pause");
            }
            onStreamUrlError: {
                if (nextAction.enabled) {
                    mediaView.nextTrack();
                }
            }
            onCommentsReply: {
                if (commentDataModel.size() == 0) {
                    commentDataModel.isPopulated = false;
                    commentTimer.stop();
                } else {
                    commentDataModel.isPopulated = true;
                    commentTimer.interval = 200;
                    commentTimer.start();
                }
            }
        },
        PostActions {
            id: postActions
        },
        NowPlayingConnection {
            id: nowPlayingConnect
            connectionName: "SoundNine"
            overlayStyle: OverlayStyle.Fancy
            onAcquired: {
                audioPlayer.play()
                var metadata = {
                    "album": "SoundNine",
                    "title": nowPlaying.title,
                    "artist": nowPlaying.user.username
                };
                nowPlayingConnect.duration = audioPlayer.duration;
                nowPlayingConnect.position = audioPlayer.position;
                nowPlayingConnect.mediaState = MediaState.Started;
                nowPlayingConnect.setMetaData(metadata);
            }
            onPause: {
                audioPlayer.pause();
                playAction.setImageSource("asset:///images/playButtonx81.png");
                playAction.setTitle("Play");
            }
            onPlay: {
                audioPlayer.play();
                playAction.setImageSource("asset:///images/pauseButtonx81.png");
                playAction.setTitle("Pause");
            }
            onRevoked: {
                audioPlayer.pause()
                playAction.setImageSource("asset:///images/playButtonx81.png");
                playAction.setTitle("Play");
            }
            onPrevious: {
                previousTrack();
            }
            onNext: {
                nextTrack();
            }
        },
        ArrayDataModel {
            id: commentDataModel
            property bool isPopulated: false
            objectName: "commentDataModel"
        },
        ArrayDataModel {
            id: commentQueue
        },
        QTimer {
            id: commentTimer
            property int lastCommentLoadedTime: -1
            property int timeToShowLastComment
            singleShot: true
            interval: 200
            onTimeout: {
                if (commentQueue.size() > 0) {
                    var nextComment = commentQueue.value(0);
                    commentQueue.removeAt(0);
                    var commentString = nextComment.body;
                    commentBodyLabel.setText(commentString);
                    commentAuthorLabel.setText(nextComment.user.username);
                    commentAvatarImage.setUrl(nextComment.user.avatar_url);
                    if (! commentBox.hidden && ! commentBox.visible) {
                        commentBox.setVisible(true);
                    }

                    if (commentQueue.size() > 1) {
                        if (commentString.length > 100)
                            timeToShowLastComment = 100 * 55;
                        else
                            timeToShowLastComment = commentString.length * 55; //45

                    } else if (commentQueue.size() == 1) {
                        timeToShowLastComment = 2000;
                    }
                    commentTimer.interval = timeToShowLastComment;
                    commentTimer.start();
                } else if (commentQueue.size() == 0) {
                    if (! commentBox.hidden && commentBox.visible)
                        commentBox.setVisible(false);
                    commentTimer.interval = 200;
                    commentTimer.start();
                }
            }
        }
    ]
    onStreamUrlChanged: {
        //console.log("streamUrl - " + streamUrl);
        feedHttp.getStreamUrl(streamUrl);
        commentTimer.stop();
        commentQueue.clear();
        commentBox.visible = false;
        feedHttp.getComments(nowPlaying.id, commentDataModel);
        nowPlayingConnect.trackChange();
        app.downloadImage(nowPlaying.artwork_url, nowPlayingConnect);
        var metadata = {
            "title": nowPlaying.title,
            "artist": nowPlaying.user.username,
            "album": "SoundNine"
        };
        nowPlayingConnect.setMetaData(metadata);
    }
    function nextTrack() {
        var nextUp = currentDB.data([ currentIndex + 1 ]);
        var type = nextUp.type;
        if (type != undefined && type.indexOf("playlist") != -1 || nextUp.playlist != null) {
            if (nextAction.enabled == true) {
                currentIndex ++;
                if (currentDB.data([ index + 2 ]) == null) {
                    nextAction.setEnabled(false);
                    nowPlayingConnect.setNextEnabled(false);
                } else {
                    nextAction.setEnabled(true);
                    nowPlayingConnect.setNextEnabled(true);
                }
                nextTrack();
                return;
            }
        }
        if (nextUp.origin) {
            nowPlaying = nextUp.origin;
        } else if (nextUp.track) {
            nowPlaying = nextUp.track;
        } else {
            nowPlaying = nextUp;
        }
        currentIndex ++;
        if (currentDB.data([ index + 2 ]) == null) {
            nextAction.setEnabled(false);
            nowPlayingConnect.setNextEnabled(false);
        } else {
            nextAction.setEnabled(true);
            nowPlayingConnect.setNextEnabled(true);
        }
        if (nowPlaying.streamable == false) {
            app.showStreamableToast(nowPlaying.title);
            nextTrack();
        }
    }
    function previousTrack() {
        var nextUp = currentDB.data([ currentIndex - 1 ]);
        var type = nextUp.type;
        if (type != undefined && type.indexOf("playlist") != -1 || nextUp.playlist != null) {
            if (previousAction.enabled == true) {
                currentIndex --;
                if (index == 0) {
                    previousAction.setEnabled(false);
                    nowPlayingConnect.setPreviousEnabled(false);
                } else {
                    previousAction.setEnabled(true);
                    nowPlayingConnect.setPreviousEnabled(true);
                }
                previousTrack();
                return;
            }
        }
        if (nextUp.origin) {
            nowPlaying = nextUp.origin;
        } else if (nextUp.track) {
            nowPlaying = nextUp.track;
        } else {
            nowPlaying = nextUp;
        }
        currentIndex --;
        if (index == 0) {
            previousAction.setEnabled(false);
            nowPlayingConnect.setPreviousEnabled(false);
        } else {
            previousAction.setEnabled(true);
            nowPlayingConnect.setPreviousEnabled(true);
        }
        if (nowPlaying.streamable == false) {
            app.showStreamableToast(nowPlaying.title);
            nextTrack();
        }
    }
}
