import bb.cascades 1.0
import org.labsquare 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Container {
    horizontalAlignment: HorizontalAlignment.Fill
    topMargin: 0
    id: feedItemContainer
    property string thumbnail: getThumb()
    function getThumb() {
        if (ListItemData.playlist.artwork_url != null) {
            return ListItemData.playlist.artwork_url;
        } else {
            return ListItemData.playlist.user.avatar_url;
        }
    }
    Container {
        id: outer
        horizontalAlignment: HorizontalAlignment.Fill
        topPadding: 5
        bottomPadding: 5
        rightPadding: 5
        leftPadding: 5
        Container {
            id: inner
            horizontalAlignment: HorizontalAlignment.Fill
            topPadding: 10
            rightPadding: 10
            leftPadding: 5
            bottomPadding: 10
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }
            Container {
                layout: DockLayout {
                }
                ImageView {
                    id: thumbBackground
                    visible: false
                    imageSource: "asset:///images/playlist.png"
                    preferredHeight: 170
                    preferredWidth: 170
                    minWidth: 170
                }
                Container {
                    leftPadding: 10
                    topPadding: 10
                    WebImageView {
                        id: thumbnailView
                        preferredHeight: 150
                        preferredWidth: 150
                        minWidth: 150
                        scalingMethod: ScalingMethod.AspectFit
                        url: thumbnail
                        onLoadingFinished: {
                            thumbBackground.setVisible(true);
                        }
                    }
                }
                touchPropagationMode: TouchPropagationMode.PassThrough
            
            }
            Container {
                leftPadding: 10
                //topPadding: 3.0
                Container {
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    }
                    Label {
                        id: artistLabel
                        textStyle.fontSize: FontSize.Medium
                        textStyle.color: Color.DarkGray
                        text: ListItemData.playlist.user.username
                        bottomMargin: 0
                    }
                    Container {
                        visible: checkRepost()
                        verticalAlignment: VerticalAlignment.Center
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        ImageView {
                            verticalAlignment: VerticalAlignment.Center
                            imageSource: "asset:///images/repostIndicator.png"
                            minHeight: 25
                            minWidth: 25
                        }
                        Label {
                            textStyle.fontSize: FontSize.Medium
                            textStyle.color: Color.DarkGray
                            text: ListItemData.user.username
                        }
                        function checkRepost() {
                            if (ListItemData.type == "playlist-repost") {
                                return true
                            } else return false
                        }
                    }
                }
                Label {
                    topMargin: 0
                    bottomMargin: 5
                    id: itemTitle
                    text: ListItemData.playlist.title
                    //content.flags: TextContentFlag.ActiveText
                    textStyle.fontSize: FontSize.Medium
                    textStyle.fontWeight: FontWeight.W500
                    multiline: true
                    touchPropagationMode: TouchPropagationMode.PassThrough
                    attachedObjects: [
                        FeedHttpCalls {
                            id: feedHttp
                        }
                    ]
                }
                Container {
                    //background: Color.Yellow
                    topPadding: 0
                    topMargin: 0
                    id: subtextContainer
                    //verticalAlignment: VerticalAlignment.Center
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    
                    }
                    ImageView {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 30
                        maxHeight: 25
                        maxWidth: 25
                        imageSource: "asset:///images/tracksIndicator.png"
                    }
                    Label {
                        verticalAlignment: VerticalAlignment.Center
                        id: playCount
                        text: ListItemData.playlist.track_count
                        textStyle.fontSize: FontSize.Small
                        textStyle.color: Color.DarkGray
                        rightMargin: 20
                        topMargin: 0
                    }
                
                }
            }
        }
    }
    Divider {
        topMargin: 0
        bottomMargin: 0
    }
    property variant blueBackground: Color.create("#00A7DD")
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: {
                outer.background = blueBackground
                inner.background = blueBackground
            }
        }
    ]
    function themeStyleToString(style) {
        switch (style) {
            case VisualStyle.Bright:
                return Color.White
            case VisualStyle.Dark:
                return Color.Black
        }
        //return "Theme: UNKNOWN :("
        return Color.Black
    }
    onTouch: {
        if (event.touchType == TouchType.Down) {
            outer.background = blueBackground
            inner.background = themeStyleToString(Application.themeSupport.theme.colorTheme.style)
        } else if (event.touchType == TouchType.Up || event.touchType == TouchType.Cancel) {
            inner.background = Color.Transparent;
            outer.background = Color.Transparent;
        }
    }
    contextActions: [
        ActionSet {
            title: ListItemData.playlist.title
            subtitle: ListItemData.playlist.user.username
            ActionItem {
                id: repostAction
                title: "Repost"
                imageSource: "asset:///images/repostContextIconx81.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    postActions.repostPlaylist(selectedItem.playlist.id);
                }
            }
            ActionItem {
                ActionBar.placement: ActionBarPlacement.InOverflow
                imageSource: "asset:///images/addToPlaylist.png"
                title: "Add to Playlist"
                onTriggered: {
                    postActions.showPlaylistDialog(null, ListItemData.playlist.id);
                }
            }
            ActionItem {
                id: userAction
                title: "View user"
                imageSource: "asset:///images/user.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    var page = userView.createObject();
                    page.userID = selectedItem.playlist.user_id;
                    //page.artworkUrl = playlistRepostListItem.ListItem.view.getNowPlayingArtwork();
                    playlistRepostListItem.ListItem.view.pushPageStack(page);
                }
            }
            InvokeActionItem {
                id: shareAction
                objectName: "shareAction"
                title: "Share"
                query {
                    mimeType: "text/plain"
                    invokeActionId: "bb.action.SHARE"
                }
                onTriggered: {
                    data = ListItemData.playlist.title + " -- " + ListItemData.playlist.permalink_url + "\nvia SoundNine";
                }
            }
        }
    ]
    attachedObjects: [
        ComponentDefinition {
            id: userView
            source: "UserView.qml"
        },
        PostActions {
            id: postActions
        }
    ]
}
