import bb.cascades 1.0
import bb.system 1.0
import org.labsquare 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Container {
    horizontalAlignment: HorizontalAlignment.Fill
    topMargin: 0
    id: feedItemContainer
    property variant usersVote: ListItemData.user_favorite
    property string thumbnail: getThumb()
    function getThumb() {
        if (ListItemData.artwork_url != null) {
            return ListItemData.artwork_url;
        } else {
            return ListItemData.user.avatar_url;
        }
    }
    Container {
        id: outer
        horizontalAlignment: HorizontalAlignment.Fill
        topPadding: 5
        bottomPadding: 5
        rightPadding: 5
        leftPadding: 5
        Container {
            id: inner
            horizontalAlignment: HorizontalAlignment.Fill
            topPadding: 15
            rightPadding: 10
            leftPadding: 10
            bottomPadding: 10
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }
            Container {
                WebImageView {
                    id: thumbnailView
                    preferredHeight: 150
                    preferredWidth: 150
                    minWidth: 150
                    scalingMethod: ScalingMethod.AspectFit
                    url: thumbnail
                }
                touchPropagationMode: TouchPropagationMode.PassThrough
            
            }
            Container {
                leftPadding: 10
                //topPadding: 3.0
                Container {
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    }
                    Label {
                        id: artistLabel
                        textStyle.fontSize: FontSize.Medium
                        textStyle.color: Color.DarkGray
                        text: ListItemData.user.username
                        bottomMargin: 0
                    }
                }
                Label {
                    topMargin: 0
                    bottomMargin: 5
                    id: itemTitle
                    text: ListItemData.title
                    //content.flags: TextContentFlag.ActiveText
                    textStyle.fontSize: FontSize.Medium
                    textStyle.fontWeight: FontWeight.W500
                    multiline: true
                    touchPropagationMode: TouchPropagationMode.PassThrough
                    attachedObjects: [
                        FeedHttpCalls {
                            id: feedHttp
                        }
                    ]
                }
                Container {
                    //background: Color.Yellow
                    topPadding: 0
                    topMargin: 0
                    id: subtextContainer
                    //verticalAlignment: VerticalAlignment.Center
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    
                    }
                    ImageView {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 30
                        imageSource: "asset:///images/playIndicator.png"
                    }
                    Label {
                        verticalAlignment: VerticalAlignment.Center
                        id: playCount
                        text: ListItemData.playback_count
                        textStyle.fontSize: FontSize.Small
                        textStyle.color: Color.DarkGray
                        rightMargin: 20
                        topMargin: 0
                    }
                    ImageView {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 30
                        imageSource: "asset:///images/commentIndicator.png"
                    }
                    Label {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 0
                        id: commentCount
                        text: ListItemData.comment_count
                        textStyle.fontSize: FontSize.Small
                        textStyle.color: Color.DarkGray
                        rightMargin: 20
                    }
                    ImageView {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 30
                        imageSource: getVote() //"asset:///images/likeIndicator.png"
                        function getVote() {
                            if (usersVote) {
                                return "asset:///images/LikeIndicatorGold.png"
                            } else return "asset:///images/likeIndicator.png"
                        }
                    }
                    Label {
                        verticalAlignment: VerticalAlignment.Center
                        topMargin: 0
                        id: likeCount
                        text: ListItemData.favoritings_count
                        textStyle.fontSize: FontSize.Small
                        textStyle.color: Color.DarkGray
                        rightMargin: 20
                    }
                }
            }
        }
    }
    Divider {
        topMargin: 0
        bottomMargin: 0
    }
    property variant blueBackground: Color.create("#00A7DD")
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: {
                outer.background = blueBackground
                inner.background = blueBackground
            }
        }
    ]
    function themeStyleToString(style) {
        switch (style) {
            case VisualStyle.Bright:
                return Color.White
            case VisualStyle.Dark:
                return Color.Black
        }
        //return "Theme: UNKNOWN :("
        return Color.Black
    }
    onTouch: {
        if (event.touchType == TouchType.Down) {
            outer.background = blueBackground
            inner.background = themeStyleToString(Application.themeSupport.theme.colorTheme.style)
        } else if (event.touchType == TouchType.Up || event.touchType == TouchType.Cancel) {
            inner.background = Color.Transparent;
            outer.background = Color.Transparent;
        }
    }
    contextActions: [
        ActionSet {
            title: ListItemData.title
            subtitle: ListItemData.user.username
            ActionItem {
                id: likeAction
                title: "Like"
                imageSource: "asset:///images/Heart.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    var dir;
                    if (title == "Like") {
                        dir = "PUT"
                        selectedItem.user_favorite = true;
                    } else {
                        dir = "DELETE"
                        selectedItem.user_favorite = false;
                    }
                    playlistTrackListItem.ListItem.view.dataModel.replace(playlistTrackListItem.ListItem.indexPath, selectedItem);
                    postActions.likeTrack(selectedItem.id, dir);
                }
            }
            ActionItem {
                id: repostAction
                title: "Repost"
                imageSource: "asset:///images/repostContextIconx81.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    postActions.repostTrack(selectedItem.id);
                }
            }
            ActionItem {
                ActionBar.placement: ActionBarPlacement.InOverflow
                imageSource: "asset:///images/addToPlaylist.png"
                title: "Add to Playlist"
                onTriggered: {
                    postActions.showPlaylistDialog(null, ListItemData.id);
                }
            }
            ActionItem {
                id: userAction
                title: "View user"
                imageSource: "asset:///images/user.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    var page = userView.createObject();
                    page.userID = selectedItem.user_id;
                    //page.artworkUrl = playlistTrackListItem.ListItem.view.getNowPlayingArtwork();
                    playlistTrackListItem.ListItem.view.pushPageStack(page);
                }
            }
            ActionItem {
                id: downloadAction
                title: "Download"
                enabled: ListItemData.downloadable
                imageSource: "asset:///images/downloadIcon.png"
                onTriggered: {
                    var selectedItem = ListItemData;
                    if (selectedItem.downloadable == true) {
                        if (selectedItem.download_url != undefined) {
                            var dlUrl = selectedItem.download_url;
                        } else {
                            var dlUrl = selectedItem.uri + "/download";
                        }
                        playlistTrackListItem.ListItem.view.downloadTrack(dlUrl);
                    } else {
                        noDownloadToast.show();
                    }
                }
                attachedObjects: [
                    SystemToast {
                        id: noDownloadToast
                        body: "The person who posted this sound did not allow it to be downloaded!"
                        //"This sound cannot be downloaded!"
                    }
                ]
            }
            InvokeActionItem {
                id: shareAction
                objectName: "shareAction"
                title: "Share"
                query {
                    mimeType: "text/plain"
                    invokeActionId: "bb.action.SHARE"
                }
                onTriggered: {
                    data = ListItemData.title + " -- " + ListItemData.permalink_url + "\nvia SoundNine";
                }
            }
        }
    ]
    attachedObjects: [
        ComponentDefinition {
            id: userView
            source: "UserView.qml"
        },
        PostActions {
            id: postActions
        }
    ]
    contextMenuHandler: ContextMenuHandler {
        id: contextHandler
        onPopulating: {
            if (ListItemData.user_favorite == true) {
                likeAction.setTitle("Unlike");
                likeAction.setImageSource("asset:///images/HeartPressed.png");
            } else {
                likeAction.setTitle("Like");
                likeAction.setImageSource("asset:///images/Heart.png");
            }
            if (ListItemData.downloadable == true) {
                downloadAction.setTitle("Download");
            } else {
                downloadAction.setTitle("Download (not available)");
            }
        }
    }
}
