import bb.cascades 1.0
import bb.system 1.0
import org.labsquare 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Page {
    id: playlistView
    objectName: "playlistView"
    property variant playlistID
    property string title
    property string permalink
    actions: [
        ActionItem {
            ActionBar.placement: ActionBarPlacement.OnBar
            id: repostAction
            enabled: false
            title: "Repost"
            imageSource: "asset:///images/repostContextIconx81.png"
            onTriggered: {
                postActions.repostPlaylist(playlistID);
            }
        },
        InvokeActionItem {
            ActionBar.placement: ActionBarPlacement.OnBar
            id: shareAction
            enabled: false
            objectName: "shareAction"
            title: "Share"
            query {
                mimeType: "text/plain"
                invokeActionId: "bb.action.SHARE"
            }
            onTriggered: {
                data = playlistView.title + " -- " + permalink + "\nvia SoundNine";
            }
        }
    ]
    titleBar: TitleBar {
        id: titleBar
        title: qsTr("Set")
        kind: TitleBarKind.FreeForm
        scrollBehavior: TitleBarScrollBehavior.Sticky
        kindProperties: FreeFormTitleBarKindProperties {
            id: expandableTitlebar
            content: Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                background: Color.create("#FF6500")
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    topPadding: 25
                    leftPadding: 15
                    bottomPadding: 10
                    preferredWidth: maxWidth

                    Label {
                        text: "Set"
                        textStyle.fontSize: FontSize.Large
                        textStyle.color: Color.White
                    }
                }

            }
            expandableArea.expanded: false
            expandableArea.toggleArea: TitleBarExpandableAreaToggleArea.IndicatorOnly
            expandableArea.content: Container {
                Container {
                    Container {

                        preferredWidth: maxWidth
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        background: Color.create('#CDCDCD')
                        WebImageView {
                            id: thumbnailViewAccount
                            visible: true
                            preferredHeight: 200
                            preferredWidth: 200
                            minWidth: 200
                            scalingMethod: ScalingMethod.AspectFit
                            //url: thumbnailUrl
                        }
                        Container {
                            id: userInfoLabelsContainer
                            visible: false
                            topPadding: 10
                            leftPadding: 10
                            Label {
                                id: usernameLabel
                                textStyle.color: Color.Black
                                textStyle.fontWeight: FontWeight.Bold
                                bottomMargin: 0
                            }
                            Label {
                                id: fullnameLabel
                                textStyle.color: Color.Black
                                topMargin: 5
                                bottomMargin: 0
                            }
                            Container {
                                topPadding: 20
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                ImageView {
                                    verticalAlignment: VerticalAlignment.Center
                                    topMargin: 30
                                    imageSource: "asset:///images/tracksIndicator.png"
                                }
                                Label {
                                    verticalAlignment: VerticalAlignment.Center
                                    topMargin: 0
                                    id: tracksCount
                                    textStyle.fontSize: FontSize.Small
                                    textStyle.color: Color.DarkGray
                                    rightMargin: 20
                                }
                            }
                        }
                    } //account header container
                    Container {
                        background: Color.DarkGray
                        bottomMargin: 10
                        topMargin: 0
                        preferredHeight: 2
                        preferredWidth: maxWidth
                    }
                    preferredWidth: maxWidth
                    background: Color.create('#CDCDCD')
                    topPadding: 0
                    bottomPadding: 0
                }
            }
        }
    }
    Container {
        Container {
            id: listContainer
            layout: DockLayout {
            }
            ListView {
                scrollRole: ScrollRole.Main
                objectName: "feedList"
                id: feedList
                listItemComponents: [
                    ListItemComponent {
                        id: listItemRoot
                        type: "track"
                        PlaylistTrackListItem {
                            id: playlistTrackListItem
                        }
                    },
                    ListItemComponent {
                        type: "none"
                        Container {
                            visible: false
                        }
                    }
                ] //ListItemComponents
                function itemType(data, indexPath) {
                    var currentItem = dataModel.data(indexPath);
                    if (currentItem.kind == "track") {
                        return "track"
                    } else return "none";
                }
                onDataModelChanged: {
                    if (dataModel.size() <= 0) {
                        emptyLabel.setVisible(true);
                    } else {
                        emptyLabel.setVisible(false);
                    }
                }
                function downloadTrack(url) {
                    console.log("feedView dl url -- " + url);
                    feedHttp.getDownloadUrl(url);
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    if (selectedItem.streamable == false) {
                        playlistView.parent.parent.parent.showStreamableToast(selectedItem.title);
                        return;
                    }
                    playlistView.parent.parent.parent.currentDB = dataModel;
                    playlistView.parent.parent.parent.currentIndex = indexPath[0];
                    //playlistView.parent.parent.parent.nowPlaying = selectedItem;
                    nowPlaying = selectedItem;
                    playlistView.parent.parent.parent.mRemovePageParent(playlistView.parent.parent.parent.mediaView);
                    playlistView.parent.push(playlistView.parent.parent.parent.mediaView);
                }
                function pushPageStack(pageParam) {
                    playlistView.parent.push(pageParam);
                }
            }
            ActivityIndicator {
                objectName: "indicator"
                id: activity
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabel
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onPlaylistInfoReply: {
                thumbnailViewAccount.setUrl(artworkP);
                tracksCount.setText(trackCountP);
                usernameLabel.setText(usernameP);
                fullnameLabel.setText(titleP);
                playlistView.title = titleP;
                playlistView.permalink = permalinkP;
                userInfoLabelsContainer.setVisible(true);
                expandableTitlebar.expandableArea.setExpanded(true);
                repostAction.setEnabled(true);
                shareAction.setEnabled(true);
            }
        },
        PostActions {
            id: postActions
        }
    ]
    onPlaylistIDChanged: {
        feedHttp.getPlaylistTracks(playlistID, activity, feedList);
    }
}
