import bb.cascades 1.0
import bb.system 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0
import org.labsquare 1.0

Page {
    id: searchView
    property string searchTerm
    property bool allowAtEndSignal
    titleBar: TitleBar {
        id: titleBar
        scrollBehavior: TitleBarScrollBehavior.Sticky
        title: qsTr("Search")
        kind: TitleBarKind.FreeForm
        kindProperties: FreeFormTitleBarKindProperties {
            content: Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                background: Color.create("#FF6500")
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    topPadding: 25
                    leftPadding: 15
                    bottomPadding: 10
                    preferredWidth: maxWidth

                    Label {
                        text: "Search"
                        textStyle.fontSize: FontSize.Large
                        textStyle.color: Color.White
                    }
                }
                Container {
                    horizontalAlignment: HorizontalAlignment.Right
                    verticalAlignment: VerticalAlignment.Center
                    rightPadding: 15
                    Container {
                        layout: DockLayout {
                        }
                        Container {
                            id: outer
                            background: Color.Transparent
                            topPadding: 3
                            bottomPadding: 3
                            rightPadding: 3
                            leftPadding: 4
                            WebImageView {
                                property variant blueBackground: Color.create("#00A7DD")
                                horizontalAlignment: HorizontalAlignment.Right
                                verticalAlignment: VerticalAlignment.Center
                                id: nowPlayingThumb
                                preferredHeight: 85
                                preferredWidth: 85
                                minWidth: 85
                                enabled: false
                                scalingMethod: ScalingMethod.AspectFit
                                url: nowPlaying.artwork_url
                                onTouch: {
                                    if (enabled) {
                                        if (event.touchType == TouchType.Down) {
                                            outer.background = blueBackground;
                                        } else if (event.touchType == TouchType.Up) {
                                            outer.background = Color.Transparent;
                                            app.removePageParent(mediaView);
                                            navPane3.push(mediaView);
                                        } else if (event.touchType == TouchType.Cancel) {
                                            outer.background = Color.Transparent;
                                        }
                                    }
                                }
                                onUrlChanged: {
                                    if (url) {
                                        enabled = true;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    Container {
        Container {
            bottomPadding: 10
            preferredWidth: maxWidth
            background: Color.create('#CDCDCD')
            Container {
                topPadding: 10
                bottomPadding: 10
                leftPadding: 10
                rightPadding: 10
                TextField {
                    input {
                        submitKey: SubmitKey.Search
                        onSubmitted: {
                            feedHttp.getSearchResults(segControl.selectedOption.text, activity, feedList, text);
                        }
                    }
                    id: textField
                    horizontalAlignment: HorizontalAlignment.Center
                    verticalAlignment: VerticalAlignment.Center
                    hintText: "Search SoundCloud..."
                }
            }
            Divider {
                topMargin: 0
                bottomMargin: 10
            }
            SegmentedControl {
                id: segControl
                horizontalAlignment: HorizontalAlignment.Center
                verticalAlignment: VerticalAlignment.Center
                options: [
                    Option {
                        text: "Sounds"
                    },
                    Option {
                        text: "People"
                    }
                ]
                onSelectedOptionChanged: {
                    if (textField.text != "")
                        feedHttp.getSearchResults(selectedOption.text, activity, feedList, textField.text);
                }
            }
        }
        Container {
            id: listContainer
            layout: DockLayout {
            }
            ListView {
                scrollRole: ScrollRole.Main
                objectName: "feedList"
                id: feedList
                listItemComponents: [
                    ListItemComponent {
                        id: peopleListItem
                        type: "user"
                        SearchPeopleListItem {

                        }
                    },
                    ListItemComponent {
                        id: listItemRoot
                        type: "sound"
                        SearchSoundListItem {
                            id: feedListItem

                        }
                    },
                    ListItemComponent {
                        id: footerListItem
                        type: "footer"
                        Container {
                            layout: StackLayout {
                                orientation: LayoutOrientation.LeftToRight
                            }
                            id: footerContainer
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            ActivityIndicator {
                                running: true
                                preferredWidth: 100
                                preferredHeight: 100
                                horizontalAlignment: HorizontalAlignment.Center
                                verticalAlignment: VerticalAlignment.Center
                            }
                            Label {
                                verticalAlignment: VerticalAlignment.Center
                                text: "loading more"
                            }
                        }
                    }
                ] //ListItemComponents
                function itemType(data, indexPath) {
                    var currentItem = dataModel.data(indexPath);
                    if (currentItem.kind == "user") {
                        return "user";
                    } else if (currentItem.type == "footer") {
                        return "footer";
                    } else {
                        return "sound"
                    }
                }
                onDataModelChanged: {
                    if (dataModel.size() <= 0) {
                        allowAtEndSignal = false;
                        emptyLabel.setVisible(true);
                    } else {
                        allowAtEndSignal = true;
                        emptyLabel.setVisible(false);
                    }
                }
                function pushPageStack(pageParam) {
                    navPane3.push(pageParam);
                }
                function getNowPlayingArtwork() {
                    if (nowPlaying) {
                        return nowPlaying.artwork_url;
                    } else return ""
                }
                function downloadTrack(url) {
                    feedHttp.getDownloadUrl(url);
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    if (selectedItem.kind != "user") { //sound is selected
                        if (selectedItem.streamable == false) {
                            app.showStreamableToast(selectedItem.title);
                            return;
                        }
                        currentDB = dataModel;
                        currentIndex = indexPath[0];
                        if (nowPlaying == undefined) {
                            navPane3.push(mediaView);
                        }
                        if (selectedItem.artwork_url == null) {
                            selectedItem.artwork_url = selectedItem.user.avatar_url;
                        }
                        nowPlaying = selectedItem;
                    } else { //user is selected
                        var page = userView.createObject();
                        page.userID = selectedItem.id;
                        //page.artworkUrl = getNowPlayingArtwork();
                        navPane3.push(page);
                    }
                    function getNowPlayingArtwork() {
                        if (nowPlaying) {
                            return nowPlaying.artwork_url;
                        } else return ""
                    }
                }

                attachedObjects: [
                    ComponentDefinition {
                        id: userView
                        source: "UserView.qml"
                    },
                    ListScrollStateHandler {
                        onAtEndChanged: {
                            if (atEnd == true && allowAtEndSignal == true) {
                                allowAtEndSignal = false;
                                feedHttp.getNextPage(feedList);
                            }
                        }
                    },
                    ComponentDefinition {
                        id: playlistView
                        source: "PlaylistView.qml"
                    }
                ]
            }
            ActivityIndicator {
                objectName: "indicator"
                id: activity
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabel
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        } //list container
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onUserInfoReply: {
                thumbnailView.setUrl(avatarP);
                tracksCount.setText(tracksP);
                followersCount.setText(followersP);
                usernameLabel.setText(usernameP);
                fullnameLabel.setText(nameP);
                userInfoLabelsContainer.setVisible(true);
            }
            onNextPageFinished: {
                allowAtEndSignal = true;
            }
        }
    ]
    onSearchTermChanged: {
        textField.setText(searchTerm);
        feedHttp.getSearchResults(segControl.selectedOption.text, activity, feedList, searchTerm);
    }
}
