import bb.cascades 1.0
import bb.system 1.0
import org.labsquare 1.0
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Page {
    id: userView
    property bool allowAtEndSignal
    property string userID
    actions: [
        ActionItem {
            ActionBar.placement: ActionBarPlacement.OnBar
            id: followAction
            enabled: false
            title: "Follow"
            imageSource: "asset:///images/user.png"
            onTriggered: {
                if (title == "Follow") {
                    feedHttp.changeFollow(userID, "PUT");
                    setTitle("Following");
                    setImageSource("asset:///images/unfollowIcon.png");
                } else {
                    feedHttp.changeFollow(userID, "DELETE");
                    setTitle("Follow");
                    setImageSource("asset:///images/followIcon.png");
                }
            }
        }
    ]
    titleBar: TitleBar {
        id: titleBar
        title: qsTr("User")
        kind: TitleBarKind.FreeForm
        scrollBehavior: TitleBarScrollBehavior.Sticky
        kindProperties: FreeFormTitleBarKindProperties {
            id: expandableTitlebar
            content: Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                background: Color.create("#FF6500")
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    topPadding: 25
                    leftPadding: 15
                    bottomPadding: 10
                    preferredWidth: maxWidth

                    Label {
                        text: "User"
                        textStyle.fontSize: FontSize.Large
                        textStyle.color: Color.White
                    }
                }
            }
            expandableArea.expanded: false
            expandableArea.toggleArea: TitleBarExpandableAreaToggleArea.IndicatorOnly
            expandableArea.content: Container {
                Container {
                    onCreationCompleted: {
                        //feedHttp.getUserInfo(userID, activity);
                    }
                    Container {

                        preferredWidth: maxWidth
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        background: Color.create('#CDCDCD')
                        WebImageView {
                            id: thumbnailViewAccount
                            visible: true
                            preferredHeight: 200
                            preferredWidth: 200
                            minWidth: 200
                            scalingMethod: ScalingMethod.AspectFit
                            //url: thumbnailUrl
                        }
                        Container {
                            id: userInfoLabelsContainer
                            visible: false
                            topPadding: 10
                            leftPadding: 10
                            Label {
                                id: usernameLabel
                                textStyle.color: Color.Black
                                textStyle.fontWeight: FontWeight.Bold
                                bottomMargin: 0
                            }
                            Label {
                                id: fullnameLabel
                                textStyle.color: Color.Black
                                topMargin: 5
                                bottomMargin: 0
                            }
                            Container {
                                topPadding: 20
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                ImageView {
                                    verticalAlignment: VerticalAlignment.Center
                                    topMargin: 30
                                    imageSource: "asset:///images/followersIndicatorx25.png"
                                }
                                Label {
                                    verticalAlignment: VerticalAlignment.Center
                                    id: followersCount
                                    //text: ListItemData.user.followers_count
                                    textStyle.fontSize: FontSize.Small
                                    textStyle.color: Color.DarkGray
                                    rightMargin: 20
                                    topMargin: 0
                                }
                                ImageView {
                                    verticalAlignment: VerticalAlignment.Center
                                    topMargin: 30
                                    imageSource: "asset:///images/tracksIndicator.png"
                                }
                                Label {
                                    verticalAlignment: VerticalAlignment.Center
                                    topMargin: 0
                                    id: tracksCount
                                    //text: ListItemData.user.track_count
                                    textStyle.fontSize: FontSize.Small
                                    textStyle.color: Color.DarkGray
                                    rightMargin: 20
                                }
                            }
                        }
                    } //account header container
                    Container {
                        background: Color.DarkGray
                        bottomMargin: 10
                        topMargin: 0
                        preferredHeight: 2
                        preferredWidth: maxWidth
                    }
                    preferredWidth: maxWidth
                    background: Color.create('#CDCDCD')
                    topPadding: 0
                    bottomPadding: 10
                    SegmentedControl {
                        id: segControl
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        options: [
                            Option {
                                text: "Sounds"
                            },
                            Option {
                                text: "Likes"
                            },
                            Option {
                                text: "Info"
                            }
                        ]
                        onSelectedOptionChanged: {
                            if (selectedIndex < 2) {
                                feedHttp.getUserTracks("users/" + userID, selectedOption.text, activity, feedList);
                                infoPage.setVisible(false);
                                listContainer.setVisible(true);
                            } else {
                                infoPage.setVisible(true);
                                listContainer.setVisible(false);
                            }
                        }
                    }
                }
            }
        }
    }
    Container {
        Container {
            id: listContainer
            layout: DockLayout {
            }
            ListView {
                scrollRole: ScrollRole.Main
                objectName: "feedList"
                id: feedList
                listItemComponents: [
                    ListItemComponent {
                        id: listItemRoot
                        type: "track"
                        AccountListItem {
                            id: accountListItem
                        }
                    },
                    ListItemComponent {
                        id: playlistComponent
                        type: "playlist"
                        PlaylistListItem {
                            id: playlistListItem
                        }
                    },
                    ListItemComponent {
                        id: footerListItem
                        type: "footer"
                        Container {
                            layout: StackLayout {
                                orientation: LayoutOrientation.LeftToRight
                            }
                            id: footerContainer
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            ActivityIndicator {
                                running: true
                                preferredWidth: 100
                                preferredHeight: 100
                                horizontalAlignment: HorizontalAlignment.Center
                                verticalAlignment: VerticalAlignment.Center
                            }
                            Label {
                                verticalAlignment: VerticalAlignment.Center
                                text: "loading more"
                            }
                        }
                    },
                    ListItemComponent {
                        type: "none"
                        Container {
                            visible: false
                        }
                    }
                ] //ListItemComponents
                function itemType(data, indexPath) {
                    var currentItem = dataModel.data(indexPath);
                    if (currentItem.playlist != null) {
                        return "playlist";
                    } else if (currentItem.track != null) {
                        return "track"
                    } else if (currentItem.type == "footer") {
                        return "footer";
                    } else return "none";
                }
                onDataModelChanged: {
                    if (dataModel.size() <= 0) {
                        allowAtEndSignal = false;
                        emptyLabel.setVisible(true);
                    } else {
                        allowAtEndSignal = true;
                        emptyLabel.setVisible(false);
                    }
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    if (selectedItem.playlist != null) {
                        var playlistFeed = playlistView.createObject();
                        playlistFeed.playlistID = selectedItem.playlist.id;
                        userView.parent.push(playlistFeed);
                    } else {
                        if (selectedItem.track.streamable == false) {
                            userView.parent.parent.parent.showStreamableToast(selectedItem.track.title);
                            return;
                        }
                        userView.parent.parent.parent.currentIndex = -1;
                        //userView.parent.parent.parent.nowPlaying = selectedItem.track;
                        nowPlaying = selectedItem.track;
                        userView.parent.parent.parent.mRemovePageParent(userView.parent.parent.parent.mediaView);
                        userView.parent.push(userView.parent.parent.parent.mediaView);
                    }
                }
                function pushPageStack(pageParam) {
                    userView.parent.push(pageParam);
                }
                function getCurrentID() {
                    return userID;
                }
                attachedObjects: [
                    ListScrollStateHandler {
                        onAtEndChanged: {
                            if (atEnd == true && allowAtEndSignal == true) {
                                allowAtEndSignal = false;
                                //feedHttp.getNextPageUserTracks("users/" + userID, segControl.selectedOption.text, feedList);
                                feedHttp.getNextPage(feedList);
                            }
                        }
                    },
                    ComponentDefinition {
                        id: playlistView
                        source: "PlaylistView.qml"
                    }
                ]
            }
            ActivityIndicator {
                objectName: "indicator"
                id: activity
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabel
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        } //list container
        Container {
            id: infoPage
            visible: false
            ListView {
                id: infoList
                dataModel: XmlDataModel {
                    source: "models/moreAccountOptions.xml"
                }
                listItemComponents: ListItemComponent {
                    type: "listItem"
                    StandardListItem {
                        title: ListItemData.title
                    }
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    var page = userListPage.createObject();
                    page.title = selectedItem.title
                    page.option = selectedItem.urlT;
                    page.userID = userID;
                    userView.parent.push(page);
                }
                attachedObjects: [
                    ComponentDefinition {
                        id: userListPage
                        source: "UserListPage.qml"
                    }
                ]
            }
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onUserInfoReply: {
                thumbnailViewAccount.setUrl(avatarP);
                tracksCount.setText(tracksP);
                followersCount.setText(followersP);
                usernameLabel.setText(usernameP);
                fullnameLabel.setText(nameP);
                userInfoLabelsContainer.setVisible(true);
                expandableTitlebar.expandableArea.setExpanded(true);
            }
            onCheckFollowValue: {
                if (followingP) {
                    followAction.setTitle("Following")
                    followAction.setImageSource("asset:///images/unfollowIcon.png");
                } else {
                    followAction.setTitle("Follow")
                    followAction.setImageSource("asset:///images/followIcon.png");
                }
                followAction.setEnabled(true);
            }
            onNextPageFinished: {
                allowAtEndSignal = true;
            }
        }
    ]
    onUserIDChanged: {
        feedHttp.checkFollow(userID);
        feedHttp.getUserInfo(userID, activity);
        feedHttp.getUserTracks("users/" + userID, "sounds", activity, feedList);
    }
}
