import bb.cascades 1.0

TabbedPane {
    id: root
    property bool loggedIn: false
    property variant currentDB
    property int currentIndex
    property variant nowPlaying
    property variant mediaView
    showTabsOnActionBar: true
    activeTab: feedTab
    Menu.definition: MenuDefinition {
        settingsAction: SettingsActionItem {
            title: qsTr("Logout")
            imageSource: "asset:///images/logout.png"
            onTriggered: {
                //if (nowPlaying != null) {
                if (nowPlaying != null) {
                    mediaView.audioPlayer.stop();
                    //should revoke the nowPlayingConnection here...
                }
                app.logout();
                loggedIn = false;
                var loginPage = loginView.createObject();
                loginPage.open();
            }
        }
        helpAction: HelpActionItem {
            imageSource: "asset:///images/ic_info.png"
            title: qsTr("About")
            onTriggered: {
                aboutPage.open();
            }
        }
        actions: ActionItem {
            title: "Invite"
            imageSource: "asset:///images/ic_bbm.png"
            onTriggered: {
                app.inviteUserToDownloadViaBBM();
            }
        }
        attachedObjects: [
            Sheet {
                id: aboutPage
                AboutPage {
                    onBack: {
                        aboutPage.close();
                    }
                }
            }
        ]
    }
    Tab {
        id: feedTab
        title: qsTr("Stream")
        imageSource: "asset:///images/tracksButton.png"
        NavigationPane {
            id: navPane
            FeedView {
            }
            onPopTransitionEnded: {
                if (page != mediaView && page.objectName != "playlistView") {
                    page.destroy();
                }
            }
        }
    }
    Tab {
        title: qsTr("Account")
        imageSource: "asset:///images/user.png"
        NavigationPane {
            id: navPane2
            AccountView {
            }
            onPopTransitionEnded: {
                if (page != mediaView && page.objectName != "playlistView") {
                    page.destroy();
                }
            }
        }
    }
    Tab {
        id: searchTab
        title: qsTr("Search")
        imageSource: "asset:///images/ic_search.png"
        NavigationPane {
            id: navPane3
            SearchView {
            	id: searchView
            }
            onPopTransitionEnded: {
                if (page != mediaView && page.objectName != "playlistView") {
                    page.destroy();
                }
            }
        }
    }
    Tab {
        title: qsTr("Activity")
        imageSource: "asset:///images/activityIcon.png"
        NavigationPane {
            id: navPane4
            ActivityView {
            }
            onPopTransitionEnded: {
                if (page != mediaView && page.objectName != "playlistView") {
                    page.destroy();
                }
            }
        }
    }
    attachedObjects: [
        ComponentDefinition {
            id: mediaViewComponent
            source: "MediaView.qml"
        },
        ComponentDefinition {
            id: loginView
            source: "LoginView.qml"
        }
    ]
    function mRemovePageParent(page) {
        //console.log("remove page parent");
        app.removePageParent(page);
    } 
    function showStreamableToast(title) {
        app.showStreamableToast(title);
    }
    function openSearch(searchString) {
        root.activeTab = searchTab;
        searchView.searchTerm = searchString;
    }
    onCreationCompleted: {
        loggedIn = app.getLoggedIn();
        if (! loggedIn) {
            var loginPage = loginView.createObject();
            loginPage.open();
        }
        mediaView = mediaViewComponent.createObject();
        Application.thumbnail.connect(onThumbnail);
        app.invokeSearch.connect(openSearch);
    }
    function onThumbnail() {
        var thumbSize = nowPlaying.artwork_url;
        thumbSize = thumbSize.replace("large", "t500x500");
        activeFrame.update(thumbSize, nowPlaying.title, nowPlaying.user.username);
    }
    onNowPlayingChanged: {
        root.onThumbnail()
    }
}